﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Data.EntityClient;
using System.Data;
using System.Data.Objects;
using System.Data.Common;

namespace ConsoleApplication1
{
    class Program
    {
        static void Main(string[] args)
        {
            DeleateAll();
//            CreateData();
            QueryByLinq();
            //        QueryByEntitySql();
            //        QueryByQueryBuilder();
            //        QueryByEntityClient();
            ;
        }

        private static void DeleateAll()
        {
            using (var container = new AddressBookContainer()) //1. オブジェクトコンテキストの生成
            {
                container.ExecuteStoreCommand("DELETE from EntryCategory;"); //2. ネイティブSQLによるデータの削除
                container.ExecuteStoreCommand("DELETE from Entries;");
                container.ExecuteStoreCommand("DELETE from Categories;");
            }
        }

        private static void QueryByLinq()
        {
            using (var container = new AddressBookContainer()) //1. オブジェクトコンテキストの生成
            {
                string name = "Doi";
                var entries = from entry in container.Entries where entry.Name == name select entry ; //2. LINQ to Entitiesでクエリを実行
                PrintEntries(entries); //3. クエリ結果を表示する
            }
        }

        private static void PrintEntries(IQueryable<Entry> entries)
        {
            foreach (var entry in entries) //6. 取得したエントリを順に表示する
            {
                Console.WriteLine(string.Format("{0} {1}歳 電話番号:{2}", entry.Name, entry.Age, entry.TelNo));
//                entry.Categories.Load();
                foreach (var cat in entry.Categories) //7. エントリに関連づけられたカテゴリも表示する
                {
                    Console.WriteLine(string.Format(" カテゴリ名:{0}", cat.CategoryName));
                }
            }
        }

        private static void QueryByEntityClient()
        {
            using (EntityConnection cn = new EntityConnection("Name=AddressBookContainer"))
            {
                cn.Open();
                using (EntityCommand cmd = new EntityCommand("SELECT VALUE c FROM AddressBookContainer.Entries AS c WHERE c.Name = @name", cn))
                {
                    cmd.Parameters.AddWithValue("name", "Doi");
                    using (EntityDataReader rdr
                          = cmd.ExecuteReader(CommandBehavior.SequentialAccess))
                    {
                        while (rdr.Read())
                        {
                            var name = rdr.GetValue(1);
                            var telno = rdr.GetValue(3);
                            var age = rdr.GetValue(4);
                            Console.WriteLine(string.Format("{0} {1}歳 電話番号:{2}", name, age, telno));
                        }
                    }
                }
            }
        }

        private static void QueryByEntitySql()
        {
            using (var container = new AddressBookContainer()) //1. オブジェクトコンテキストの生成
            {
                string query = "SELECT VALUE a FROM AddressBookContainer.Entries AS a "
                            + "WHERE a.Name = @name"; //2. Entity SQL文字列の設定
                var parameters = new System.Data.Objects.ObjectParameter[1]; //3. Entity SQLに与えるパラメータの設定
                parameters[0] = new System.Data.Objects.ObjectParameter("Name", "Doi");
                var entries = container.CreateQuery<Entry>(query, parameters); //4. CreateQueryメソッドでクエリを呼び出す
                PrintEntries(entries); //5. クエリ結果を表示する
            }
        }

        private static void QueryByQueryBuilder()
        {
            using (var conatiner = new AddressBookContainer()) //1. オブジェクトコンテキストの生成
            {
                var entries = conatiner.Entries.Where("it.Age > @age", new ObjectParameter("age", 20)); //2. Query builderのWhereメソッドを呼び出す
                PrintEntries(entries); //3. クエリ結果を表示する
            }
        }

        private static void CreateData()
        {
            using (var container = new AddressBookContainer())
            {
                var entry1 = new Entry()
                {
                    Name = "Doi",
                    Age = 30,
                    TelNo = "03-0000-0000",
                    MailAddress = "doi@hogehoge.com"
                }; //1. Entryエンティティを作成
                var entry2 = new Entry()
                {
                    Name = "Tanaka",
                    Age = 20,
                    TelNo = "043-000-0000",
                    MailAddress = "tanaka@hoge.com"
                };

                var category1 = new Category()
                {
                    CategoryName = "友達"
                }; //2. Categoryエンティティを作成
                var category2 = new Category()
                {
                    CategoryName = "仕事"
                };
                var category3 = new Category()
                {
                    CategoryName = "頻繁に連絡"
                };

                entry1.Categories.Add(category1); //3. EntryエンティティにCategoryエンティティを追加
                entry1.Categories.Add(category3);
                entry2.Categories.Add(category2);
                entry2.Categories.Add(category3);

                container.AddToEntries(entry1); //4. Entryエンティティをデータベースに追加
                container.AddToEntries(entry2);
                container.SaveChanges(); //5. 変更内容を保存
            }
        }
    }
}
